import requests
import urllib3
from urllib3.exceptions import InsecureRequestWarning
urllib3.disable_warnings(InsecureRequestWarning)
from pathlib import Path
import zipfile
import os
from arelle import Cntlr, ModelManager
from arelle.ModelValue import qname

# 書類一覧から書類管理番号を取得する関数
def get_doc_id(date, edinet_code, doc_type):

    # EDINETのAPIを使って書類一覧を取得する
    url = 'https://disclosure.edinet-fsa.go.jp/api/v1/documents.json'
    params = {
    "date": date,
    "type": 2,
    }
    
    docs = requests.get(url, params=params, verify=False)
    docs_results = docs.json()['results']
    
    for result in docs_results:
        if result['docTypeCode'] != doc_type: # 書類種別コードで抽出
            continue
        if result['edinetCode'] == edinet_code: # EDINETコードで抽出
            doc_id = result['docID'] # 書類管理番号を取得
    
    return doc_id

# 指定した書類管理番号の書類を取得し、zip形式で保存
def download_zip(date, edinet_code, doc_id):

    # EDINETのAPIを使って書類をダウンロードする
    url = f'https://disclosure.edinet-fsa.go.jp/api/v1/documents/{doc_id}'
    params = {
    "type": 1
    }

    binary_response = requests.get(url, params=params, verify=False)
    
    # 有価証券報告書を保存する「yuho_report」フォルダーを作る
    base_dir = Path(__file__).resolve().parent
    save_dir_name = 'yuho_report'
    if not os.path.exists(Path(base_dir, save_dir_name)):
        os.makedirs(Path(base_dir, save_dir_name))
    
    # 書類のファイル名を「日付_EDINETコード」に設定する
    file_path = f'{base_dir}/{save_dir_name}/{date}_{edinet_code}'
    # 書類をzip形式で保存する
    with open(f'{file_path}.zip', 'wb') as f:
        for chunk in binary_response.iter_content(chunk_size=1024):
            f.write(chunk)
    
    # zipファイルを解凍する
    with zipfile.ZipFile(f'{file_path}.zip') as existing_zip:
        existing_zip.extractall(file_path)
    
    return file_path

# 日付、EDINETコード、書類種別コードを指定して書類管理番号を取得する
date = '2022-06-30'
edinet_code = 'E02367' # 「任天堂」
doc_type = '120' # 「有価証券報告書」

doc_id = get_doc_id(date, edinet_code, doc_type)
file_path = download_zip(date, edinet_code, doc_id)

# Arelleを使って売上高を抽出するコード
ctrl = Cntlr.Cntlr()
model_manager = ModelManager.initialize(ctrl)
xbrl_file_path = f'{file_path}/XBRL/PublicDoc/jpcrp030000-asr-001_E02367-000_2022-03-31_01_2022-06-30.xbrl'

model_xbrl = model_manager.load(xbrl_file_path)
ns = model_xbrl.prefixedNamespaces["jppfs_cor"]
qn = qname(ns,name="jppfs_cor_NetSales")
facts = model_xbrl.factsByQname[qn]

for fact in model_xbrl.facts:
    jp_label = fact.concept.label()
    if jp_label == "売上高" and fact.concept.qname.localName == "NetSalesSummaryOfBusinessResults" and (not "NonConsolidatedMember" in fact.contextID):
        print(fact)

model_manager.close()